#include "kwikstik.h"
#include "Driver_SLCD.h"
#include "Sound_Recorder.h"
#include "Remote_Control.h"
#include "Driver_Electrodes.h"
#include <event.h>
/**
  Local Function Prototypes
*/
void _MainMenu_Parse(pointer electrode_event);

/**
  Global Constants
*/
const STRING Introduction = {"KWIKSTIK"};
const STRING Select = {"SEL."};
const STRING Right = {"RGT"};
const STRING Left = {"LFT"};

const STRING Options[4] =
{
  "SOUND RECORDER",
  "REMOTE CONTROL",
  "USB MOUSE",
  "FRETRIS"
};

/**
  Global Variables
*/
unsigned char MainMenuStateMachine = INIT;
U08 OptionIndex = 0;
S16 MarqueePosition = 256;
BOOL isMarqueeInitialDelay = TRUE;
BOOL isLoopComplete = FALSE;


/**
  Function Name	: Main_Task
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011

  Parameters	: uint_32 data
  Returns       : Nothing
  Notes         : This is the main application function. It initializes the electrodes and LCD
                  and installs the used interrupts. If a touch event is reported it will either
                  jump to the selected application or navigate through the menu. After all the initialization
                  is completed it falls into a simple state machine.
*/
void Main_Task(uint_32 data)
{
  //unsigned short IntroductionPosition = 300;
  //boolean IntroductionEnd = FALSE;
  pointer  electrode_event;


  _int_disable();

  //Install interrupt for FlexTimer2 (used by Remote Control)
  _int_install_isr(INT_FTM2, _FlexTimer_FTM2_ISR, NULL);
  _cortex_int_init(INT_FTM2, 2, TRUE);

  //Install interrupt for FlexTimer0 (used by Sound Recorder)
  _int_install_isr(INT_FTM0, _FlexTimer_FTM0_ISR, NULL);
   _cortex_int_init(INT_FTM0, 4, TRUE);

  //Install interrupt for PITO (used by Buzzer)
  _int_install_isr(INT_PIT0, _PIT0_Isr, NULL);
  _cortex_int_init(INT_PIT0, 4, TRUE);

  _int_enable();

  //Initialize Buzzer
  _Buzzer_Init();

  //Initialize the Segment LCD
  _SLCDModule_Init();

  //Initailze the Touch Sense Software
  _Electrodes_Init();
  TSS_KeypadConfig(cKey0.ControlId,Keypad_MaxTouches_Register,1);
  TSS_SetSystemConfig(System_ElectrodeEnablers_Register,0x3F); 

  //Print out "KwikStik" to LCD screen
  _SLCDModule_TurnOffAllSegments();
  _SLCDModule_PrintScrolling(Introduction);
  _time_delay(500);

  //Turn on and then off all the segments
  _SLCDModule_TurnOnAllSegments();
  _Buzzer_On(100);
  _time_delay(1000);
  _SLCDModule_TurnOffAllSegments();

  //Create task to monitor battery level and which USB ports are being used
  _task_create(0,STATUS_TASK,0);

  //Turn on FSL icon on segment LCD
  _SLCDModule_TurnOnFreescaleSign();
  
   /* open event connection */
   if (_event_open("event.global",&electrode_event) != MQX_OK)  
   {
      printf("\nOpen Event failed");
      _mqx_exit(0);
   }  
      
  MainMenuStateMachine = MAIN;
  while(1)
  {
    switch (MainMenuStateMachine)
    {
      case INIT:

        break;
      case MAIN:
        /**
          Applications showed
        */
        TSS_SetSystemConfig(System_ElectrodeEnablers_Register,0x33); 
        _SLCDModule_PrintString(Left, 9);
        _SLCDModule_PrintRCBlocks(UPPER_LEFT);
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString(Right, 9);
        _SLCDModule_PrintRCBlocks(UPPER_RIGHT);
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString(Select, 9);
        _SLCDModule_PrintRCBlocks(LOWER_RIGHT);
        _SLCDModule_PrintRCBlocks(LOWER_LEFT);
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintLeftArrow();
        _SLCDModule_PrintRightArrow();
        OptionIndex = 0;

        isMarqueeInitialDelay = TRUE;
        MarqueePosition = 258;
        MainMenuStateMachine = ELECTRODE_CHECK;
        break;

      case ELECTRODE_CHECK:
        _MainMenu_Parse(electrode_event);
      break;

      case SOUND_RECORDER:
      break;
   
      case REMOTE_CONTROLLER:
      break;

      case USB_MOUSE:
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString("MOUSE", 4);

        while(MainMenuStateMachine!=MAIN)
        {
          _time_delay(100);
        }
      break;
  
      case FRETRIS:

        while(MainMenuStateMachine!=MAIN)
        {
          _time_delay(100);
        }
      break;
      
      default:
      break;
    }

    _time_delay(1);
  }
}

/**
  Function Name	: _MainMenu_Parse
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011

  Parameters: electrode_event point to the event group storing touch events
  Returns: None
  Notes         : This function controls the main menu for the KwikStik demo.
                  The function displays the name of the application selected
                  by the current menu index. The function also monitors the
                  status of the electrode events. If a touch occurs it takes
                  the appropriate action (incrementing the index to the next
                  application, decrementing the index to the previous 
                  application, or selecting the current application. If the 
                  current application is selected, then the state machine is
                  updated so that the task for the selected application will
                  start processing.
*/
void _MainMenu_Parse(pointer electrode_event)
{
//  static U08 MarqueeDelay = 0x9;
  uint_32 electrode_value;
  
  if (_event_get_value(electrode_event, &electrode_value) != MQX_OK)  
  {
    printf("\nGet Event failed");
    _mqx_exit(0);
  }

  /* Check to see if the current application is being selected */
  if ((electrode_value & TSS_ELECTRODE_4) || (electrode_value & TSS_ELECTRODE_5))
  {
    switch (OptionIndex)
    {
      case 0:
        MainMenuStateMachine = SOUND_RECORDER;
        ElectrodeFlags = 0;
        /* Clear all event flags */
        _event_clear(electrode_event, 0x3F);
        _Buzzer_On(40);
        break;
      case 1:
        MainMenuStateMachine = REMOTE_CONTROLLER;
        ElectrodeFlags = 0;
        /* Clear all event flags */
        _event_clear(electrode_event, 0x3F);
        _Buzzer_On(40);
        break;
      case 2:
        MainMenuStateMachine = USB_MOUSE;
        ElectrodeFlags = 0;
        /* Clear all event flags */
        _event_clear(electrode_event, 0x3F);
        _Buzzer_On(40);
        break;
      case 3:
        MainMenuStateMachine = FRETRIS;
        ElectrodeFlags = 0;
        /* Clear all event flags */
        _event_clear(electrode_event, 0x3F);
        _Buzzer_On(40);
        break;      
      default:
        break;
    }
  }
  /* Test to see if the menu should decrement to the previous application */
  else if (electrode_value & TSS_ELECTRODE_0)
  {
    /* Decrement or wrap the application index */
    if(OptionIndex != 0)
      OptionIndex--;
    else
      OptionIndex = MENU_OPTIONS;
      
    MarqueePosition = 258;
    isMarqueeRestart = TRUE;
    isMarqueeInitialDelay = TRUE;
    ElectrodeFlags = 0;
    /* Clear all event flags */
    _event_clear(electrode_event, 0x3F);
    _Buzzer_On(40);
  }
  /* Test to see if the menu should increment to the next application */
  else if (electrode_value & TSS_ELECTRODE_1)
  {
    /* Increment or wrap the application index */
    if(OptionIndex != MENU_OPTIONS)
      OptionIndex++;
    else
      OptionIndex = 0;
    
    MarqueePosition = 258;
    isMarqueeRestart = TRUE;
    isMarqueeInitialDelay = TRUE;
    ElectrodeFlags = 0;
    /* Clear all event flags */
    _event_clear(electrode_event, 0x3F);
    _Buzzer_On(40);
  }
  
  /* No touch detected, so continue with the display of the current application option */

  /* Brief pause before displaying the application name */
  if (isMarqueeInitialDelay == TRUE)
  {
    _time_delay(200);
    isMarqueeInitialDelay = FALSE;
  }

  /* Until the entire name has been displayed scroll a marquee across the LCD */
  if ((_SLCDModule_PrintMarquee(Options[OptionIndex],MarqueePosition--,MARQUEE_WITH_ARROWS,MARQUEE_WITH_ARROWS)) == 1)
  {
    MarqueePosition = 258;
    isLoopComplete = TRUE;
  }

  /* The entire name has been displayed, so put flags back to initial values */
  if (isLoopComplete == TRUE)
  {
    isMarqueeInitialDelay = TRUE;
    isLoopComplete = FALSE;
  }
  

  /* Pause for a while, but break out of the pause if a touch occurs */
  _event_wait_any(electrode_event, 0x3F, 50); 
}