#include "Sound_Recorder.h"
#include <event.h>

/** 
  Enumeration for the Sound Recorder State Machine.
*/
enum 
{
  SOUND_INIT = 0,
  SOUND_MAIN,
  RECORDING,
  PLAYING,
  EXIT
}eSoundRecorder;

STRING Seconds = {"SEC"};
STRING Record = {"REC"};
STRING Play = {"PLAY"};
STRING Exit = {"EXIT"};

/**
  Global Variables
*/
U16 ADCMicrophoneValues[16000] = {0};
U08 SoundRecorderStateMachine = SOUND_INIT;
volatile BOOL isTakeSample = FALSE;
volatile BOOL isWriteSample = FALSE;
U08 SoundMenuIndex = 0;
volatile BOOL isRecording = FALSE;
volatile BOOL isPlaying = FALSE;

/**
  Function Name	: _SoundRecorder_Task
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011
  
  Parameters	: Nothing
  Returns       : Nothing
  Notes         : This is the sound recorder application function. It initializes the FlexTimer, GPIO's, ADC and DAC needed.
                  After that it falls into a simple state machine, where it will remain until the break condition
                  is met. While no touch event occurs it will keep showing its execution options. This application only
                  records / plays 2 seconds because of memory limitations. 
*/
void _SoundRecorder_Task(uint_32 param)
{   
   pointer electrode_event;

   /* open event connection */
   if (_event_open("event.global",&electrode_event) != MQX_OK)  
   {
      printf("\nOpen Event failed");
      _mqx_exit(0);
   }
  
  while (1)
  {    
    //Wait for SoundRecorder mode to be entered
    while(MainMenuStateMachine!=SOUND_RECORDER)
    {
      _time_delay(500);
    }

    switch(SoundRecorderStateMachine)
    {
      case SOUND_INIT:
        ElectrodeFlags = 0;
        _ADC0_Init(10, CHANNEL_A, MODE_12);
        _DAC1_Init();
        _FlexTimer_Init(FTM0,DIV_BY_16,INTERRUPT_ENABLE,375);  
        _GPIO_Init(PORTE,28);
        _GPIO_Init(PORTD,10);
        _GPIO_Set_Pin(PORTD,10);       
        (void)TSS_SetSystemConfig(System_ElectrodeEnablers_Register,0x32);
        (void)TSS_KeypadConfig(cKey0.ControlId,Keypad_MaxTouches_Register,1);      
  
        _SLCDModule_ClearLCD(NO_ARROWS);  
        
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString(Record, 7);
        _SLCDModule_PrintRCBlocks(UPPER_RIGHT);
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString(Play, 7);
        _SLCDModule_PrintRCBlocks(LOWER_RIGHT);          
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
        _SLCDModule_PrintString(Exit, 7);
        _SLCDModule_PrintRCBlocks(LOWER_LEFT);          
        _time_delay(800);
        _SLCDModule_ClearLCD(NO_ARROWS);
  
        SoundMenuIndex = 0;
        
        isPlaying = FALSE;
        isRecording = FALSE;
        
        SoundRecorderStateMachine = SOUND_MAIN;     
      break;
      
      case SOUND_MAIN:
        if (SoundMenuIndex == 0)
        {
          _SLCDModule_PrintString(Record, 7);
          _SLCDModule_PrintRCBlocks(UPPER_RIGHT);
          SoundMenuIndex++;
        }
        else if (SoundMenuIndex == 1)
        {
          _SLCDModule_ClearLCD(NO_ARROWS);
          _SLCDModule_PrintString(Play, 7);
          _SLCDModule_PrintRCBlocks(LOWER_RIGHT);          
          SoundMenuIndex++;
        }
        else if (SoundMenuIndex == 2)
        {
          _SLCDModule_ClearLCD(NO_ARROWS);
          _SLCDModule_PrintString(Exit, 7);
          _SLCDModule_PrintRCBlocks(LOWER_LEFT);       
          SoundMenuIndex++;
        }
        else if (SoundMenuIndex == 3)
        {              
          _SLCDModule_ClearLCD(NO_ARROWS);
          SoundMenuIndex = 0;
        }
        
        if (!(_event_wait_any(electrode_event, 0x3F, 1000) != MQX_OK)) 
          /* touch event happened to escape the wait loop, so process
           * the touch to determine what action to take. 
           */
          _SoundRecorder_Menu_Parse(electrode_event);
      break;      
    
      case RECORDING:
        while(isRecording)
        {
          _SoundRecorder_Start_Recording();
          ElectrodeFlags = 0;
          /* Clear all event flags */
          _event_clear(electrode_event, 0x3F);
        }
      break;
    
      case PLAYING:
        while(isPlaying)
        {
          _SoundRecorder_Start_Playing();
          ElectrodeFlags = 0;
          /* Clear all event flags */
          _event_clear(electrode_event, 0x3F);
        }
      break;
    
      case EXIT:
        _ADC0_Disable();
        _DAC1_Disable();  
        isPlaying = FALSE;
        isRecording = FALSE;
        MainMenuStateMachine = MAIN;
        _SLCDModule_ClearLCD(NO_ARROWS);
        (void)TSS_SetSystemConfig(System_ElectrodeEnablers_Register,0x3F);
        SoundRecorderStateMachine = INIT;
        isMarqueeRestart = TRUE;
        SoundMenuIndex = 0;
        ElectrodeFlags = 0;
        /* Clear all event flags */
        _event_clear(electrode_event, 0x3F);
      break;
    
      default: 
      break;
    }
  }    
}     

/**
  Function Name	: _SoundRecorder_Start_Recording
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011
  
  Parameters	: Nothing
  Returns       : Nothing
  Notes         : This function starts to record and save the ADC values to a buffer. Once the
                  buffer size is reached it exits. It displays on the LCD the remaining time.
*/
void _SoundRecorder_Start_Recording(void)
{
  static U16 RecordingSampleCount = 0;
  static U16 SecondsTimer = 0;
  static U16 Time1 = 1;
  static U16 Time2 = 0;
  
  if(isTakeSample)
  {
    ADCMicrophoneValues[RecordingSampleCount++] = _ADC0_Read(10,CHANNEL_A);
    isTakeSample = FALSE;
    SecondsTimer++;
    if (SecondsTimer == ONE_SECOND)
    {
      SecondsTimer = 0;
      _SLCDModule_PrintNumber(Time1,6);      
      _SLCDModule_PrintNumber(Time2,0);
      Time1++;          
      if (Time1 > 8)
      {
        Time1 = 0;
        //Time2++;
        if (Time2 > 9)
        {
          //Time2 = 0;
        }
      }
    }
  }
  if (RecordingSampleCount < (sizeof(ADCMicrophoneValues)/2))
  {
    //Nothing
  }
  else
  {
    _FlexTimer_Disable(FTM0);
    RecordingSampleCount = 0;
    _time_delay(200);
    _SLCDModule_ClearLCD(NO_ARROWS);    
    SoundRecorderStateMachine = SOUND_MAIN; 
    Time1 = 1;
    isRecording = FALSE;
  }
}

/**
  Function Name	: _SoundRecorder_Start_Playing
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011
  
  Parameters	: Nothing
  Returns       : Nothing
  Notes         : This function starts to play the values saved on the buffer. Once it reaches
                  the buffer size it exits. It displays on the LCD the remainig time.
*/
void _SoundRecorder_Start_Playing(void)
{
  static U16 PlayingSampleCount = 0;
  static U16 SecondsTimer = 0;
  static U16 Time1 = 1;
  static U16 Time2 = 0; 
  
  _FlexTimer_Enable(FTM0);
  _DAC1_Enable();
  _GPIO_Set_Pin(PORTE,28);

  if(isWriteSample)
  {
    _DAC1_Write(ADCMicrophoneValues[PlayingSampleCount++]);
    isWriteSample = FALSE;
    SecondsTimer++;
    if (SecondsTimer == ONE_SECOND)
    {
      _SLCDModule_PrintNumber(Time1,6);
      _SLCDModule_PrintNumber(Time2,0);
      if (Time1 != 0)
      {
        Time1--;
      }
      else
      {
        //Time1 = 9;
        //Time2--;
      }        
      SecondsTimer = 0;
    } 
  }    
  if (PlayingSampleCount < (sizeof(ADCMicrophoneValues)/2))
  {
  }
  else
  {
    _FlexTimer_Disable(FTM0);   
    _DAC1_Disable();
    PlayingSampleCount = 0;
    _GPIO_Clear_Pin(PORTE,28);
    Time1 = 1;
    _time_delay(200);
    _SLCDModule_ClearLCD(NO_ARROWS);
    isPlaying = FALSE;
    SoundRecorderStateMachine = SOUND_MAIN;
  }
}

/**
  Function Name	: _SoundRecorder_Menu_Parse
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011
  
  Parameters: electrode_event point to the event group storing touch events
  Returns: None
  Notes         : This function is used process touch events after one has been
                  detected. The function will determine which electrode was
                  touched, take the appropriate action (setting the state
                  machine to record, play or exit, then clear all event flags.
*/
void _SoundRecorder_Menu_Parse(pointer electrode_event)
{
  uint_32 electrode_value;
  
  
  if (_event_get_value(electrode_event, &electrode_value) != MQX_OK)  
  {
    printf("Get Event failed");
    _mqx_exit(0);
  }
  
  if (electrode_value & TSS_ELECTRODE_1)
  {
    SoundRecorderStateMachine = RECORDING;
    _FlexTimer_Enable(FTM0);
    _SLCDModule_ClearLCD(NO_ARROWS);
    _SLCDModule_PrintNumber(0,0);
    _SLCDModule_PrintNumber(0,6);
    _SLCDModule_PrintString(Seconds,16);
    _Buzzer_On(40);
    while(isBuzzerOn){}    
    //_time_delay(40);
    isRecording = TRUE;
  }
  if (electrode_value & TSS_ELECTRODE_5)
  {
    SoundRecorderStateMachine = PLAYING;
    _FlexTimer_Enable(FTM0);           
    _SLCDModule_ClearLCD(NO_ARROWS);
    _SLCDModule_PrintNumber(0,0);
    _SLCDModule_PrintNumber(2,6);    
    _SLCDModule_PrintString(Seconds,16);    
    _Buzzer_On(40);
    while(isBuzzerOn){}    
    //_time_delay(40);
    isPlaying = TRUE;
  }
  if (electrode_value & TSS_ELECTRODE_4)
  {
    SoundRecorderStateMachine = EXIT;
    _Buzzer_On(40);    
    while(isBuzzerOn){}
    //_time_delay(40);
  }
}

/**
  Function Name	: _FlexTimer_FTM0_ISR
  Engineer      : Manglio Gonzlez Carrasco
  Date          : 05/01/2011
  
  Parameters	: pointer isr
  Returns       : Nothing
  Notes         : This is the interrupt service routine of the Flextimer. It just
                  sets two boolean flags. Its current period is 8 KHz.
*/
void _FlexTimer_FTM0_ISR(pointer isr)
{
  isWriteSample = TRUE;
  isTakeSample =  TRUE;
  FTM0_SC &= ~(1 << FTM_SC_TOF_SHIFT);
}
